@@ -57,13 +57,6 @@
 #define	MAX_IDENT_SIZE		(MAX_USERID_SIZE + 50)
 #define	MAX_URI_SIZE		256
 
-#define	FWD_ALL_ENABLED		0x01
-#define	FWD_BUSY_ENABLED	0x02
-#define	FWD_BNA_ENABLED		0x04
-#define	FWD_DND_ENABLED		0x08
-#define	FWD_AWAY_ENABLED	0x10
-#define	FWD_PUBLIC_ENABLED	0x20
-
 NAMESPACE_SIPWITCH
 using namespace UCOMMON_NAMESPACE;
 
