@@ -29,6 +29,14 @@
 #include <sipwitch/service.h>
 #endif
 
+#ifndef	__SIPWITCH_CDR_H_
+#include <sipwitch/cdr.h>
+#endif
+
+#ifndef	_SIPWITCH_PROCESS_H_
+#include <sipwitch/process.h>
+#endif
+
 NAMESPACE_SIPWITCH
 using namespace UCOMMON_NAMESPACE;
 
@@ -59,6 +67,9 @@ public:
 	protected:
 		generic();
 	};
+
+	static void cdrlog(FILE *fp, cdr *call);	
+	static void errlog(errlevel_t level, const char *text);
 };
 
 END_NAMESPACE
