@@ -180,36 +180,6 @@ bool server::classify(rtpproxy::session *sid, rtpproxy::session *src, struct soc
 	return rtn;
 }
 
-unsigned server::forwarding(keynode *leaf)
-{
-	unsigned mask = 0;
-	const char *cp;
-
-	if(!leaf)
-		return 0;
-
-	cp = service::getValue(leaf, "all");
-	if(cp && *cp)
-		mask |= FWD_ALL_ENABLED;
-	cp = service::getValue(leaf, "busy");
-	if(cp && *cp)
-		mask |= FWD_BUSY_ENABLED;
-	cp = service::getValue(leaf, "bna");
-	if(cp && *cp)
-		mask |= FWD_BNA_ENABLED;
-	cp = service::getValue(leaf, "dnd");
-	if(cp && *cp)
-		mask |= FWD_DND_ENABLED;
-	cp = service::getValue(leaf, "away");
-	if(cp && *cp)
-		mask |= FWD_AWAY_ENABLED;
-	cp = service::getValue(leaf, "public");
-	if(cp && *cp)
-		mask |= FWD_PUBLIC_ENABLED;
-
-	return mask;
-}
-
 service::keynode *server::find(const char *id)
 {
 	assert(id != NULL && *id != 0);
@@ -257,6 +227,7 @@ void server::confirm(const char *user)
 	char filename[65];
 	caddr_t mp;
 	profile *pp, *ppd;
+	const char *state = root.getPointer();
 	const char *realm = registry::getRealm();
 	unsigned prefix = registry::getPrefix();
 	unsigned range = registry::getRange();
@@ -326,10 +297,18 @@ void server::confirm(const char *user)
 	process::errlog(DEBUG1, "scanning config from %s", dirpath);
 	while(is(dir) && fsys::read(dir, filename, sizeof(filename)) > 0) {
 		ext = strrchr(filename, '.');
-		if(ext && stricmp(ext, ".xml"))
+		if(!ext || !String::equal(ext, ".xml"))
 			continue;
-		snprintf(buf, sizeof(buf), "%s/%s", dirpath, filename);
-		fp = fopen(buf, "r");
+		if(state) {
+			snprintf(buf, sizeof(buf), "%s/%s/%s", dirpath, state, filename);
+			fp = fopen(buf, "r");
+		}
+		else
+			fp = NULL;
+		if(!fp) {
+			snprintf(buf, sizeof(buf), "%s/%s", dirpath, filename);
+			fp = fopen(buf, "r");
+		}
 		fn = strrchr(buf, '/');
 		if(fn)
 			++fn;
@@ -570,18 +549,6 @@ service::keynode *server::getRouting(const char *id)
 	locking.release();
 	return NULL;
 }
-	
-unsigned server::getForwarding(const char *uid)
-{
-	assert(uid != NULL);
-
-	usernode user;
-	getProvision(uid, user);
-	unsigned mask = forwarding(user.keys);
-	
-	server::release(user);
-	return mask;
-}
 
 void server::getDialing(const char *uid, usernode& user)
 {
@@ -685,6 +652,7 @@ void server::reload(const char *uid)
 
 	char buf[256];
 	FILE *state = NULL;
+	const char *cp;
 
 #ifdef _MSWINDOWS_
 	GetEnvironmentVariable("APPDATA", buf, 192);
@@ -714,6 +682,10 @@ void server::reload(const char *uid)
 			return;
 		}
 
+	cp = cfgp->root.getPointer();
+	if(cp)
+		process::errlog(INFO, "activating for state \"%s\"", cp);
+
 	cfgp->commit(uid);
 	if(!cfg) {
 		process::errlog(FAILURE, "no configuration");
